@@ -69,39 +69,44 @@ function showControls()
 
 function setLabelForNode(aNode, aLabel, aIsLabelFlag)
 {
-  // This is for labels with possible accesskeys
-  var accessKeyIndex;
-  if (/^\&[^\&]/.test(aLabel)) { // access key is at the start
-   accessKeyIndex = 0;
-  } else {
-    accessKeyIndex = aLabel.search(/[^\&]\&[^\&]/) + 1;
-    if (accessKeyIndex == 0) {
-      accessKeyIndex = -1; // magic value for no accesskey
-    }
+  // This is for labels which may contain embedded access keys.
+  // If we end in (&X) where X represents the access key, optionally preceded
+  // by spaces and/or followed by the ':' character, store the access key and
+  // remove the access key placeholder + leading spaces from the label.
+  // Otherwise a character preceded by one but not two &s is the access key.
+  // Store it and remove the &.
+
+  // Note that if you change the following code, see the comment of
+  // nsTextBoxFrame::UpdateAccessTitle.
+  var accessKey = null;
+  if (/ *\(\&([^&])\)(:)?$/.test(aLabel)) {
+    aLabel = RegExp.leftContext + RegExp.$2;
+    accessKey = RegExp.$1;
+  } else if (/^(.*[^&])?\&(([^&]).*$)/.test(aLabel)) {
+    aLabel = RegExp.$1 + RegExp.$2;
+    accessKey = RegExp.$3;
   }
 
-  // If a character has an & before it, then it should become an accesskey
-  if (accessKeyIndex >= 0 && accessKeyIndex < aLabel.length - 1) {
-    // This will also cause the accesskey attribute to be set via xbl
-    aNode.accessKey = aLabel.charAt(accessKeyIndex + 1);
-    // Set the label to the string without the &
-    aLabel = aLabel.substr(0, accessKeyIndex) + 
-             aLabel.substr(accessKeyIndex + 1);
-  }
+  // && is the magic sequence to embed an & in your label.
   aLabel = aLabel.replace(/\&\&/g, "&");
   if (aIsLabelFlag) {    // Set text for <label> element
     aNode.setAttribute("value", aLabel);
   } else {    // Set text for other xul elements
     aNode.label = aLabel;
   }
+
+  // XXXjag bug 325251
+  // Need to set this after aNode.setAttribute("value", aLabel);
+  if (accessKey)
+    aNode.accessKey = accessKey;
 }
 
 function commonDialogOnLoad()
 {
   // set the document title
-//@line 105 "/builds/tinderbox/XR-Mozilla1.8.0-Release/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/content/commonDialog.js"
+//@line 110 "/builds/tinderbox/XR-Mozilla1.8/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/content/commonDialog.js"
   document.title = gCommonDialogParam.GetString(12);
-//@line 107 "/builds/tinderbox/XR-Mozilla1.8.0-Release/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/content/commonDialog.js"
+//@line 112 "/builds/tinderbox/XR-Mozilla1.8/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/content/commonDialog.js"
 
   // set the number of command buttons
   var nButtons = gCommonDialogParam.GetInt(2);
@@ -117,16 +122,9 @@ function commonDialogOnLoad()
   }
 
   // display the main text
-  var messageText = gCommonDialogParam.GetString(0);
-  var messageParent = document.getElementById("info.box");
-  var messageParagraphs = messageText.split("\n");
-
-  for (var i = 0; i < messageParagraphs.length; i++) {
-    var descriptionNode = document.createElement("description");
-    var text = document.createTextNode(messageParagraphs[i]);
-    descriptionNode.appendChild(text);
-    messageParent.appendChild(descriptionNode);
-  }
+  var messageParent = document.getElementById("info.box").getElementsByTagName('description')[0];
+  // XXX the substr(0, 10000) part is a workaround for bug 317334
+  messageParent.textContent = gCommonDialogParam.GetString(0).substr(0, 10000);
 
   setElementText("info.header", gCommonDialogParam.GetString(3), true);
 
@@ -167,10 +165,12 @@ function commonDialogOnLoad()
   {
     var dlgButtons = ['accept', 'cancel', 'extra1', 'extra2'];
 
-    // Set the default button and focus it
+    // Set the default button and focus it on non-OS X systems
     var dButton = dlgButtons[gCommonDialogParam.GetInt(5)];
     document.documentElement.defaultButton = dButton;
+//@line 174 "/builds/tinderbox/XR-Mozilla1.8/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/content/commonDialog.js"
     document.documentElement.getButton(dButton).focus();
+//@line 176 "/builds/tinderbox/XR-Mozilla1.8/Linux_2.4.21-32.0.1.ELsmp_Depend/mozilla/toolkit/content/commonDialog.js"
   }
 
   if (gCommonDialogParam.GetInt(6) != 0) // delay button enable
